/*
  ESP32-CAM Remote Control
*/

#include "esp_wifi.h"
#include "esp_camera.h"
#include "img_converters.h"
#include <WiFi.h>

const char* ssid = "FSM";
const char* password = "0101010101";


//
// WARNING!!! Make sure that you have either selected ESP32 Wrover Module,
//            or another board which has PSRAM enabled
//

// Select camera model
#define CAMERA_MODEL_AI_THINKER

#if defined(CAMERA_MODEL_AI_THINKER)
#define PWDN_GPIO_NUM     32
#define RESET_GPIO_NUM    -1
#define XCLK_GPIO_NUM      0
#define SIOD_GPIO_NUM     26
#define SIOC_GPIO_NUM     27

#define Y9_GPIO_NUM       35
#define Y8_GPIO_NUM       34
#define Y7_GPIO_NUM       39
#define Y6_GPIO_NUM       36
#define Y5_GPIO_NUM       21
#define Y4_GPIO_NUM       19
#define Y3_GPIO_NUM       18
#define Y2_GPIO_NUM        5
#define VSYNC_GPIO_NUM    25
#define HREF_GPIO_NUM     23
#define PCLK_GPIO_NUM     22

#else
#error "Camera model not selected"
#endif

void startCameraServer();
extern camera_fb_t fb2;

void setup()
{
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  Serial.println();

  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;

  // QQVGA|CIF|QVGA|SVGA|XGA|SXGA|UXGA
  // QQVGA = 160x120, 19200bytes
  config.frame_size   = FRAMESIZE_QQVGA;
  config.pixel_format = PIXFORMAT_GRAYSCALE;
  config.fb_count = 1;

  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK)
  {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }

  //flip image
  sensor_t * s = esp_camera_sensor_get();
  s->set_vflip(s, 1);
  s->set_hmirror(s, 1);

  // Remote Control Car
  initMotors();
  initLed();

  Serial.println("ssid: " + (String)ssid);
  Serial.println("password: " + (String)password);

  WiFi.begin(ssid, password);
  delay(500);

  while (WiFi.status() != WL_CONNECTED) delay(500);

  startCameraServer();

  if (WiFi.status() == WL_CONNECTED)
  {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.print("Camera Ready! Use 'http://");
    Serial.print(WiFi.localIP());
    Serial.println("' to connect");
  }

  fb2.buf = (uint8_t *)malloc(19200);

  for (int i = 0; i < 5; i++)
  {
    ledcWrite(7, 10); // flash led
    delay(100);
    ledcWrite(7, 0);
    delay(100);
  }
}

uint8_t linebuf[160];

void loop()
{
  {
    // get the picture
    camera_fb_t *fb = esp_camera_fb_get();
    if (!fb) Serial.println("Frame buffer could not be acquired");

    Serial.printf("w: %d h: %d format: %d len: %d \n", fb->width, fb->height, fb->format, fb->len);

    // copy the picture to static buffer for webserver and processing
    fb2.width  = fb->width;
    fb2.height = fb->height;
    fb2.format = fb->format;
    fb2.len    = fb->len;
    for (int i = 0; i < fb->len; i++) fb2.buf[i] = fb->buf[i];
    esp_camera_fb_return(fb);

    // draw 2 lines
    for (int i = fb2.width * 98; i < fb2.width * 99; i++) fb2.buf[i] = 0xA0;
    for (int i = fb2.width * 103; i < fb2.width * 104; i++) fb2.buf[i] = 0xA0;

    // picture processing
    for (int i = 0; i < fb2.width; i++) linebuf[i] = fb2.buf[fb2.width * 101 + i]/4;

    for (int i = 0; i < fb2.width; i++) 
    {
      int index;
      index = fb2.width*(90-linebuf[i])+i;
      fb2.buf[index] = 0xfe;
      index += fb2.width;
      fb2.buf[index] = 0x01;
    }
    
  }

  delay(5000);
}
